from pile import Pile

## La pile globale

pile = Pile()

## Fonction récursive pour afficher une pile sans
## connaître sa représentation interne
def affiche_pile(p):
    if p.est_vide():
        print("------")
    else:
        v = p.depiler()
        print(v)
        affiche_pile(p)
        p.empiler(v)

### solution simple. On définit une fonction auxiliaire qui retire
### deux valeurs de la pile
def depiler2(p):
    """Dépile deux valeurs et les renvoie comme un couple. Le sommet
       de pile est en premier"""
    v1 = p.depiler()
    v2 = p.depiler()
    return (v1, v2)

def calculette():
    while True:
        try:
            ligne = input("Entrer un nombre ou une commande:").strip()

            if ligne == '+':
                v2, v1 = depiler2(pile) # Attention au sens
                pile.empiler(v1 + v2)
            elif ligne == '-':
                v2, v1 = depiler2(pile) # Attention au sens
                pile.empiler(v1 - v2)
            elif ligne == '*':
                v2, v1 = depiler2(pile) # Attention au sens
                pile.empiler(v1 * v2)
            elif ligne == '/':
                v2, v1 = depiler2(pile) # Attention au sens
                pile.empiler(v1 / v2)
            elif ligne == '' or ligne == 'p':
                affiche_pile(pile)
            elif ligne == 'q':
                break
            else:
                pile.empiler(int(ligne))
        except:
            print("Erreur")

## Solution alternative. On crée un dictionnaire dont
## les clés sont des symboles et les valeurs des fonctions.
def calculette2():
    ops = { '+' : lambda x,y: x+y,
            '-' : lambda x,y: x-y,
            '*' : lambda x,y: x*y,
            '/' : lambda x,y: x/y
    }
    while True:
        try:
            ligne = input("Entrer un nombre ou une commande:").strip()

            if ligne in ops: # si le symbole est une entrée du dictionnaire
                v2,v1 = depiler2(pile)
                op = ops[ligne]  # op est l'une des 4 fonctions (lambda) ci-dessus.
                pile.empiler(op(v1,v2))
            elif ligne == '' or ligne == 'p':
                affiche_pile(pile)
            elif ligne == 'q':
                break
            else:
                pile.empiler(int(ligne))
        except:
            print("Erreur")


calculette()
